from typing import TypedDict

__all__ = [
    "Shakemap",
    "EarthquakeData",
    "FeltEarthquakeData",
    "LatestEarthquakeData",
    "StrongEarthquakeData",
    "InfoFeltEarthquakeData",
    "InfoLatestEarthquakeData",
    "InfoStrongEarthquakeData",
]

Shakemap = str


class EarthquakeData(TypedDict):
    Tanggal: str
    Jam: str
    DateTime: str
    Coordinates: str
    Lintang: str
    Bujur: str
    Magnitude: str
    Kedalaman: str
    Wilayah: str


class StrongEarthquakeData(EarthquakeData):
    Potensi: str


class FeltEarthquakeData(EarthquakeData):
    Dirasakan: str


class LatestEarthquakeData(EarthquakeData):
    Potensi: str
    Dirasakan: str
    Shakemap: Shakemap


class _StrongEarthquakeData(TypedDict):
    gempa: list[StrongEarthquakeData]


class InfoStrongEarthquakeData(TypedDict):
    Infogempa: _StrongEarthquakeData


class _FeltEarthquakeData(TypedDict):
    gempa: list[FeltEarthquakeData]


class InfoFeltEarthquakeData(TypedDict):
    Infogempa: _FeltEarthquakeData


class _LatestEarthquakeData(TypedDict):
    gempa: LatestEarthquakeData


class InfoLatestEarthquakeData(TypedDict):
    Infogempa: _LatestEarthquakeData



from dataclasses import dataclass

__all__ = ["Forecast"]


@dataclass(slots=True)
class Forecast:
    """
    A schema used to store info about forecast.

    Attributes:
        domain: domain of a forecast.
    """

    domain: str



from collections.abc import Iterator
from dataclasses import dataclass
from datetime import datetime

from .area import Area
from .data import Data
from .forecast import Forecast
from .weather import Weather

__all__ = ["WeatherForecast"]


@dataclass
class WeatherForecast:
    """
    A schema used to store info about weather forecast.

    Attributes:
        data: Data schema from api.
        forecast: Forecast schema from api.
        issue: datetime of the issue.
        weathers: weather info with Area as key and iterator of Weather as value.

    Note:
        `issue` field is naive datetime, means it has no information about its
        timezone. `weathers` has an `Area` schema as it key and iterator of `Weather`
        that contain information about weather forecast of that area as it value. Only
        area type is land that has weather forecast information, whereas sea is not.
    """

    data: Data
    forecast: Forecast
    issue: datetime
    weathers: dict[Area, Iterator[Weather] | None]



from collections.abc import Iterator
from datetime import datetime
from itertools import chain

# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

# FIXME
# remove `type: ignore` if there is a stub for defusedxml
from defusedxml.ElementTree import fromstring  # type: ignore

from ..common.schemas import Coordinate
from .enums import Cardinal, Sexa, Type
from .enums import Weather as WeatherEnum
from .exception import WeatherForecastParseError
from .schemas import (
    Area,
    Data,
    Forecast,
    Humidity,
    Name,
    Temperature,
    Weather,
    WeatherForecast,
    WindDirection,
    WindSpeed,
)
from .types import (
    WeatherForecastParameter,
    WeatherForecastParameterId,
    WeatherForecastParameters,
)

__all__ = [
    "parse_data_element",
    "parse_forecast_element",
    "parse_issue_element",
    "parse_area_element",
    "parse_humidity_element",
    "parse_temperature_element",
    "parse_weather_element",
    "parse_wind_direction_element",
    "parse_wind_speed_element",
    "parse_datetime_element",
    "parse_weather_forecast_data",
]


def parse_data_element(element: Element) -> Data:
    """
    Parse data element tree in xml.

    Args:
        element: an element of data

    Returns:
        A `Data` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> element = fromstring(
    ...     '<data source="meteofactory" productioncenter="NC Jakarta"></data>'
    ... )
    >>> data = parse_data_element(element)
    >>> data
    Data(source='meteofactory', productioncenter='NC Jakarta')
    """
    source = element.get("source")
    if source is None:
        raise WeatherForecastParseError("source attribute in data tag not found")

    productioncenter = element.get("productioncenter")
    if productioncenter is None:
        raise WeatherForecastParseError(
            "productioncenter attribute in data tag not found"
        )

    return Data(source, productioncenter)


def parse_forecast_element(element: Element) -> Forecast:
    """
    Parse forecast element tree in xml.

    Args:
        element: an element of forecast

    Returns:
        A `Forecast` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> element = fromstring('<forecast domain="local"></forecast>')
    >>> forecast = parse_forecast_element(element)
    >>> forecast
    Forecast(domain='local')
    """
    domain = element.get("domain")
    if domain is None:
        raise WeatherForecastParseError("domain attribute in forecast tag not found")

    return Forecast(domain)


def parse_issue_element(element: Element) -> datetime:
    """
    Parse issue element tree in xml.

    Args:
        element: an element of issue

    Returns:
        A naive `datetime` object.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> element = fromstring(
    ...     "<issue>"
    ...     "<timestamp>20240116032347</timestamp>"
    ...     "<year>2024</year>"
    ...     "<month>01</month>"
    ...     "<day>16</day>"
    ...     "<hour>03</hour>"
    ...     "<minute>23</minute>"
    ...     "<second>47</second>"
    ...     "</issue>"
    ... )
    >>> issue = parse_issue_element(element)
    >>> issue
    datetime.datetime(2024, 1, 16, 3, 23, 47)
    """
    timestamp_element = element.find("timestamp")
    if timestamp_element is None:
        raise WeatherForecastParseError("timestamp tag in issue tag not found")

    timestamp = timestamp_element.text
    if timestamp is None:
        raise WeatherForecastParseError("timestamp tag in issue tag has no text")

    return datetime.strptime(timestamp, "%Y%m%d%H%M%S")


def parse_area_element(element: Element) -> Area:
    """
    Parse area element tree in xml.

    Args:
        element: an element of area

    Returns:
        An `Area` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> element = fromstring(
    ...     "<area"
    ...     ' id="501409"'
    ...     ' latitude="4.176594"'
    ...     ' longitude="96.124878"'
    ...     ' coordinate="96.124878 4.176594"'
    ...     ' type="land"'
    ...     ' region=""'
    ...     ' level="1"'
    ...     ' description="Aceh Barat"'
    ...     ' domain="Aceh"'
    ...     ' tags=""'
    ...     ">"
    ...     '<name xml:lang="en_US">Aceh Barat</name>'
    ...     '<name xml:lang="id_ID">Kab. Aceh Barat</name>'
    ...     "</area>"
    ... )
    >>> area = parse_area_element(element)
    >>> area
    Area(id='501409', coordinate=Coordinate(latitude=4.176594, longitude=96.124878), ...
    """
    id = element.get("id")
    if id is None:
        raise WeatherForecastParseError("id attribute in area tag not found")

    latitude = element.get("latitude")
    if latitude is None:
        raise WeatherForecastParseError("latitude attribute in area tag not found")

    longitude = element.get("longitude")
    if longitude is None:
        raise WeatherForecastParseError("longitude attribute in area tag not found")

    type = element.get("type")
    if type is None:
        raise WeatherForecastParseError("type attribute in area tag not found")

    region = element.get("region")
    if region is None:
        raise WeatherForecastParseError("region attribute in area tag not found")

    level = element.get("level")
    if level is None:
        raise WeatherForecastParseError("level attribute in area tag not found")

    description = element.get("description")
    if description is None:
        raise WeatherForecastParseError("description attribute in area tag not found")

    domain = element.get("domain")
    if domain is None:
        raise WeatherForecastParseError("domain attribute in area tag not found")

    tags = element.get("tags")
    if tags is None:
        raise WeatherForecastParseError("tags attribute in area tag not found")

    names = element.findall("name")
    if len(names) < 2:
        raise WeatherForecastParseError("one or more name tag in area tag not found")

    en_US = names[0].text
    if en_US is None:
        raise WeatherForecastParseError("name tag in area tag has no text")

    id_ID = names[1].text
    if id_ID is None:
        raise WeatherForecastParseError("name tag in area tag has no text")

    name = Name(en_US, id_ID)

    return Area(
        id,
        Coordinate(float(latitude), float(longitude)),
        Type(type),
        region,
        level,
        description,
        domain,
        tags,
        name,
    )


def parse_humidity_element(element: Element) -> Iterator[Humidity]:
    """
    Parse humidity element tree in xml.

    Args:
        element: a parameter element that contain humidity

    Yields:
        Some `Humidity` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> element = fromstring(
    ...     '<parameter id="hu" description="Humidity" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="%">95</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="%">90</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="%">95</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> humidity = parse_humidity_element(element)
    >>> humidity
    <generator object parse_humidity_element at ...>
    """
    for timerange in element:
        value_element = timerange.find("value")
        if value_element is None:
            raise WeatherForecastParseError(
                "value tag in timerange tag not found"
            ) from AttributeError

        humidity_text = value_element.text
        if humidity_text is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        yield Humidity(int(humidity_text))


def parse_temperature_element(element: Element) -> Iterator[Temperature]:
    """
    Parse temperature element tree in xml.

    Args:
        element: a parameter element that contain temperature

    Yields:
        Some `Temperature` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> element = fromstring(
    ...     '<parameter id="t" description="Temperature" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="C">24</value>'
    ...     '<value unit="F">75.2</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="C">28</value>'
    ...     '<value unit="F">82.4</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="C">26</value>'
    ...     '<value unit="F">78.8</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> temperature = parse_temperature_element(element)
    >>> temperature
    <generator object parse_temperature_element at ...>
    """
    for timerange in element:
        value_elements = timerange.findall("value")
        if len(value_elements) < 2:
            raise WeatherForecastParseError(
                "one or more value tag in timerange tag not found"
            )

        celcius = value_elements[0].text
        if celcius is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        fahrenheit = value_elements[1].text
        if fahrenheit is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        yield Temperature(float(celcius), float(fahrenheit))


def parse_weather_element(element: Element) -> Iterator[WeatherEnum]:
    """
    Parse weather element tree in xml.

    Args:
        element: a parameter element that contain weather

    Yields:
        Some `Weather` enum.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> element = fromstring(
    ...     '<parameter id="weather" description="Weather" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="icon">60</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="icon">60</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="icon">1</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> weather = parse_weather_element(element)
    >>> weather
    <generator object parse_weather_element at ...>
    """
    for timerange in element:
        value_elements = timerange.find("value")
        if value_elements is None:
            raise WeatherForecastParseError("value tag in timerange tag not found")

        weather = value_elements.text
        if weather is None:
            raise WeatherForecastParseError(
                "value tag in timerange tag not has no text"
            )

        yield WeatherEnum(int(weather))


def parse_wind_direction_element(element: Element) -> Iterator[WindDirection]:
    """
    Parse wind direction element tree in xml.

    Args:
        element: a parameter element that contain wind direction

    Yields:
        Some `WindDirection` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> element = fromstring(
    ...     '<parameter id="wd" description="Wind direction" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="deg">90</value>'
    ...     '<value unit="CARD">E</value>'
    ...     '<value unit="SEXA">9000</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="deg">157.5</value>'
    ...     '<value unit="CARD">SSE</value>'
    ...     '<value unit="SEXA">15730</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="deg">0</value>'
    ...     '<value unit="CARD">VARIABLE</value>'
    ...     '<value unit="SEXA">000</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> wind_direction = parse_wind_direction_element(element)
    >>> wind_direction
    <generator object parse_wind_direction_element at ...>
    """
    for timerange in element:
        value_elements = timerange.findall("value")
        if len(value_elements) < 3:
            raise WeatherForecastParseError(
                "one or more value tag in timerange tag not found"
            )

        deg = value_elements[0].text
        if deg is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        card = value_elements[1].text
        if card is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        sexa = value_elements[2].text
        if sexa is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        yield WindDirection(float(deg), Cardinal(card), Sexa(sexa))


def parse_wind_speed_element(element: Element) -> Iterator[WindSpeed]:
    """
    Parse wind speed element tree in xml.

    Args:
        element: a parameter element that contain wind speed

    Yields:
        Some `WindSpeed` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> element = fromstring(
    ...     '<parameter id="ws" description="Wind speed" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="Kt">5</value>'
    ...     '<value unit="MPH">5.75389725</value>'
    ...     '<value unit="KPH">9.26</value>'
    ...     '<value unit="MS">2.57222222</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="Kt">2</value>'
    ...     '<value unit="MPH">2.3015589</value>'
    ...     '<value unit="KPH">3.704</value>'
    ...     '<value unit="MS">1.028888888</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="Kt">0</value>'
    ...     '<value unit="MPH">0</value>'
    ...     '<value unit="KPH">0</value>'
    ...     '<value unit="MS">0</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> wind_speed = parse_wind_speed_element(element)
    >>> wind_speed
    <generator object parse_wind_speed_element at ...>
    """
    for timerange in element:
        value_elements = timerange.findall("value")
        if len(value_elements) < 4:
            raise WeatherForecastParseError(
                "one or more value tag in timerange tag not found"
            )

        knot = value_elements[0].text
        if knot is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        mph = value_elements[1].text
        if mph is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        kph = value_elements[2].text
        if kph is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        ms = value_elements[3].text
        if ms is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        yield WindSpeed(float(knot), float(mph), float(kph), float(ms))


def parse_datetime_element(element: Element) -> Iterator[datetime]:
    """
    Parse datetime element tree in xml.

    This parse datetime string found in element tree in the following format
    `"%Y%m%d%H%M%S"`.

    Args:
        element: any parameter element that contain datetime with type hourly

    Yields:
        Some `datetime` object.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> element = fromstring(
    ...     '<parameter id="ws" description="datetime" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="Kt">5</value>'
    ...     '<value unit="MPH">5.75389725</value>'
    ...     '<value unit="KPH">9.26</value>'
    ...     '<value unit="MS">2.57222222</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="Kt">2</value>'
    ...     '<value unit="MPH">2.3015589</value>'
    ...     '<value unit="KPH">3.704</value>'
    ...     '<value unit="MS">1.028888888</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="Kt">0</value>'
    ...     '<value unit="MPH">0</value>'
    ...     '<value unit="KPH">0</value>'
    ...     '<value unit="MS">0</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> datetime = parse_datetime_element(element)
    >>> datetime
    <generator object parse_datetime_element at ...>
    """
    for timerange in element:
        dt = timerange.get("datetime")
        if dt is None:
            raise WeatherForecastParseError(
                "datetime attribute in timerange tag not found"
            )

        yield datetime.strptime(dt, "%Y%m%d%H%M%S")


def parse_weather_forecast_data(weather_forecast_data: str | bytes) -> WeatherForecast:
    """
    Parse weather forecast data element tree in xml.

    This control when area type is sea then it has no information regarding weather
    forecast. Also this use wind speed element tree to get the datetime information.

    Args:
        weather_forecast_data: string or bytes of xml data.

    Returns:
        A` WeatherForecast` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.
    """
    root = fromstring(weather_forecast_data)

    data = parse_data_element(root)

    forecast_element = root[0]
    forecast = parse_forecast_element(forecast_element)

    issue_element = forecast_element[0]
    issue = parse_issue_element(issue_element)

    areas = {}
    for area_element in forecast_element.iterfind("area"):
        area = parse_area_element(area_element)
        # if the area type is sea, then the weather is empty
        weathers: Iterator[Weather] | None = None

        # Only land that has weather, sea doesn't
        if area.type == Type.LAND:
            parameters: WeatherForecastParameters = {}
            for parameter_element in area_element.iterfind("parameter"):
                parameter_id: WeatherForecastParameterId | None = parameter_element.get(
                    "id"
                )
                if parameter_id is None:
                    raise WeatherForecastParseError(
                        "id attribute in parameter tag not found"
                    )

                parameter: WeatherForecastParameter
                if (
                    parameter_id == "hu"
                    or parameter_id == "humax"
                    or parameter_id == "humin"
                ):
                    parameter = parse_humidity_element(parameter_element)
                elif (
                    parameter_id == "t"
                    or parameter_id == "tmax"
                    or parameter_id == "tmin"
                ):
                    parameter = parse_temperature_element(parameter_element)
                elif parameter_id == "weather":
                    parameter = parse_weather_element(parameter_element)
                elif parameter_id == "wd":
                    parameter = parse_wind_direction_element(parameter_element)
                else:
                    # parameter_id == "ws":
                    parameter = parse_wind_speed_element(parameter_element)

                    # Choose one of parameter tag that has datetime attribute with
                    # type hourly. Strictly speaking it is timerange tag.
                    parameters["datetime"] = parse_datetime_element(parameter_element)

                parameters[parameter_id] = parameter

            # The length of the data to be zipped is 12. This is because the weather
            # forecast data received is for three days, each receiving 4 data. However,
            # for tmin, tmax, hmin, and hmax data, these four data are only given 1 each
            # for that day, therefore there are only 3 data for 3 days from tmin, tmax,
            # hmin, and hmax. Because the zip() function will stop at the smallest data
            # length, therefore we need to multiply the 3 data by 4 to get 12
            weathers = (
                Weather(dt, weather, t, tmix, tmax, h, hmin, hmax, wd, ws)
                for dt, weather, t, tmix, tmax, h, hmin, hmax, wd, ws in zip(
                    parameters["datetime"],
                    parameters["weather"],
                    parameters["t"],
                    chain.from_iterable(
                        [(val, val, val, val) for val in parameters["tmin"]]
                    ),
                    chain.from_iterable(
                        [(val, val, val, val) for val in parameters["tmax"]]
                    ),
                    parameters["hu"],
                    chain.from_iterable(
                        [(val, val, val, val) for val in parameters["humin"]]
                    ),
                    chain.from_iterable(
                        [(val, val, val, val) for val in parameters["humax"]]
                    ),
                    parameters["wd"],
                    parameters["ws"],
                )
            )

        areas[area] = weathers

    return WeatherForecast(data, forecast, issue, areas)



from dataclasses import dataclass, field
from datetime import datetime

from ...common.schemas import Coordinate

__all__ = ["Earthquake"]


@dataclass(slots=True)
class Earthquake:
    """
    A schema used to store info about earthquake.

    Attributes:
        datetime: datetime of the earthquake.
        coordinate: coordinate of the earthquake.
        magnitude: magnitude is the strength of an earthquake in M as its unit.
        depth: depth of the earthquake in km as its unit.
        region: The area closest to the location of the earthquake epicenter.

    Note:
        The `datetime` field is aware datetime, meaning it has the timezone information.
    """

    datetime: datetime
    coordinate: Coordinate
    magnitude: float = field(metadata={"unit": "M"})
    depth: float = field(metadata={"unit": "km"})
    region: str



from datetime import datetime
from typing import Iterator, Literal, TypedDict

from .enums import Weather
from .schemas import (
    Humidity,
    Temperature,
    WindDirection,
    WindSpeed,
)

__all__ = [
    "WeatherForecastParameter",
    "WeatherForecastParameterId",
    "WeatherForecastParameters",
]


class WeatherForecastParameters(TypedDict, total=False):
    datetime: Iterator[datetime]
    hu: Iterator[Humidity]
    humax: Iterator[Humidity]
    humin: Iterator[Humidity]
    t: Iterator[Temperature]
    tmax: Iterator[Temperature]
    tmin: Iterator[Temperature]
    weather: Iterator[Weather]
    wd: Iterator[WindDirection]
    ws: Iterator[WindSpeed]


WeatherForecastParameter = (
    Iterator[datetime]
    | Iterator[Humidity]
    | Iterator[Temperature]
    | Iterator[Weather]
    | Iterator[WindDirection]
    | Iterator[WindSpeed]
)

WeatherForecastParameterId = Literal[
    "datetime", "hu", "humax", "humin", "t", "tmax", "tmin", "weather", "wd", "ws"
]



"""
Module to store a str enum class representation cardinal directions.
"""
from enum import StrEnum

__all__ = ["Cardinal"]


class Cardinal(StrEnum):
    """
    A str enum class that define valid cardinal direction.

    Attributes:
        NORTH: `"N"`
        NORTH_NORTHEAST: `"NNE"`
        NORTHEAST: `"NE"`
        EAST_NORTHEAST: `"ENE"`
        EAST: `"E"`
        EAST_SOUTHEAST: `"ESE"`
        SOUTHEAST: `"SE"`
        SOUTH_SOUTHEAST: `"SSE"`
        SOUTH: `"S"`
        SOUTH_SOUTHWEST: `"SSW"`
        SOUTHWEST: `"SW"`
        WEST_SOUTHWEST: `"WSW"`
        WEST: `"W"`
        WEST_NORTHWEST: `"WNW"`
        NORTHWEST: `"NW"`
        NORTH_NORTHWEST: `"NNW"`
        VARIABLE: `"VARIABLE"`

    Examples:
        >>> Cardinal("N")
        <Cardinal.NORTH: 'N'>
        >>> Cardinal["NORTH"]
        <Cardinal.NORTH: 'N'>
        >>> Cardinal.NORTH
        <Cardinal.NORTH: 'N'>
        >>> Cardinal.NORTH == "N"
        True
        >>> print(Cardinal.NORTH)
        N

    Note:
        `VARIABLE` direction mean as its name suggest, the direction can't be
        determined.
    """

    NORTH: str = "N"
    NORTH_NORTHEAST: str = "NNE"
    NORTHEAST: str = "NE"
    EAST_NORTHEAST: str = "ENE"
    EAST: str = "E"
    EAST_SOUTHEAST: str = "ESE"
    SOUTHEAST: str = "SE"
    SOUTH_SOUTHEAST: str = "SSE"
    SOUTH: str = "S"
    SOUTH_SOUTHWEST: str = "SSW"
    SOUTHWEST: str = "SW"
    WEST_SOUTHWEST: str = "WSW"
    WEST: str = "W"
    WEST_NORTHWEST: str = "WNW"
    NORTHWEST: str = "NW"
    NORTH_NORTHWEST: str = "NNW"
    VARIABLE: str = "VARIABLE"



from dataclasses import dataclass

__all__ = ["Coordinate"]


@dataclass(unsafe_hash=True, slots=True)
class Coordinate:
    """
    A schema used to store info about coordinate.

    Attributes:
        latitude: latitude of the coordinate.
        longitude: longitude of the coordinate.
    """

    latitude: float
    longitude: float



"""
Module to store common enum classes used by weather forecast.
"""
from .cardinal import Cardinal
from .province import Province
from .sexa import Sexa
from .type import Type
from .weather import Weather

__all__ = [
    "Cardinal",
    "Province",
    "Sexa",
    "Type",
    "Weather",
]



from traceback import TracebackException
from types import TracebackType
from typing import Self

from aiohttp import ClientSession


class BMKG:
    """
    Base BMKG API wrapper.
    """

    async def __aenter__(self) -> Self:
        self._session = ClientSession("https://data.bmkg.go.id")

        return self

    async def __aexit__(
        self,
        exc_type: Exception,
        exc_val: TracebackException,
        traceback: TracebackType,
    ) -> None:
        await self.close()

    async def close(self) -> None:
        await self._session.close()



from dataclasses import dataclass

from .earthquake import Earthquake

__all__ = ["StrongEarthquake"]


@dataclass(slots=True)
class StrongEarthquake:
    """
    A schema used to store info about strong earthquake.

    Attributes:
        earthquake: earthquake schema.
        potency: potential tsunami or not, and the status of the earthquake felt.
    """

    earthquake: Earthquake
    potency: str



from dataclasses import dataclass, field

__all__ = ["Humidity"]


@dataclass(slots=True)
class Humidity:
    """
    A schema used to store info about humidity.

    Attributes:
        percentage: percentage of humidity in % as its unit.
    """

    percentage: int = field(metadata={"unit": "%"})



from dataclasses import dataclass

__all__ = ["Data"]


@dataclass(slots=True)
class Data:
    """
    A schema used to store info about data.

    Attributes:
        source: source of a data.
        productioncenter: production center of a data.
    """

    source: str
    productioncenter: str



from collections.abc import Iterator
from datetime import datetime

from ..common.schemas import Coordinate
from .schemas import (
    Earthquake,
    FeltEarthquake,
    LatestEarthquake,
    StrongEarthquake,
)
from .types import (
    EarthquakeData,
    InfoFeltEarthquakeData,
    InfoLatestEarthquakeData,
    InfoStrongEarthquakeData,
)

__all__ = [
    "parse_earthquake_data",
    "parse_latest_earthquake_data",
    "parse_strong_earthquake_data",
    "parse_felt_earthquake_data",
]


def parse_earthquake_data(earthquake_data: EarthquakeData) -> Earthquake:
    """
    Parse `EarthquakeData` JSON.

    Args:
        earthquake_data: A dict representing JSON of earthquake.

    Returns:
        An `Earthquake` schema.

    Examples:
        >>> earthquake_data = EarthquakeData(
        ...     {
        ...         "Tanggal": "14 Jan 2024",
        ...         "Jam": "03:13:37 WIB",
        ...         "DateTime": "2024-01-13T20:13:37+00:00",
        ...         "Coordinates": "-6.75,106.55",
        ...         "Lintang": "6.75 LS",
        ...         "Bujur": "106.55 BT",
        ...         "Magnitude": "3.5",
        ...         "Kedalaman": "5 km",
        ...         "Wilayah": "Pusat gempa berada di darat 26 km Utara Kab Sukabumi",
        ...     }
        ... )
        >>> parse_earthquake_data(earthquake_data)
        Earthquake(datetime=datetime.datetime(2024, 1, 13, 20, 13, 37, tzinfo=datetim...
    """

    dt = earthquake_data["DateTime"]
    coordinate = earthquake_data["Coordinates"].split(",")
    latitude = coordinate[0]
    longitude = coordinate[1]
    magnitude = earthquake_data["Magnitude"]
    depth = earthquake_data["Kedalaman"].split()[0]
    region = earthquake_data["Wilayah"]

    return Earthquake(
        datetime.fromisoformat(dt),
        Coordinate(float(latitude), float(longitude)),
        float(magnitude),
        float(depth),
        region,
    )


def parse_latest_earthquake_data(
    info_latest_earthquake_data: InfoLatestEarthquakeData,
) -> LatestEarthquake:
    """
    Parse `InfoLatestEarthquakeData` JSON.

    Args:
        info_latest_earthquake_data: A dict representing JSON of info latest earthquake.

    Returns:
        A `LatestEarthquake` schema.

    Examples:
        >>> info_latest_earthquake_data = InfoLatestEarthquakeData(
        ...     {
        ...         "Infogempa": {
        ...             "gempa": {
        ...                 "Tanggal": "15 Jan 2024",
        ...                 "Jam": "16:07:08 WIB",
        ...                 "DateTime": "2024-01-15T09:07:08+00:00",
        ...                 "Coordinates": "-3.98,122.45",
        ...                 "Lintang": "3.98 LS",
        ...                 "Bujur": "122.45 BT",
        ...                 "Magnitude": "2.6",
        ...                 "Kedalaman": "9 km",
        ...                 "Wilayah": "Pusat gempa berada di darat 2,5 km BaratDaya P",
        ...                 "Potensi": "Gempa ini dirasakan untuk diteruskan pada masy",
        ...                 "Dirasakan": "II Kendari",
        ...                 "Shakemap": "20240115160708.mmi.jpg",
        ...             }
        ...         }
        ...     }
        ... )
        >>> parse_latest_earthquake_data(info_latest_earthquake_data)
        LatestEarthquake(earthquake=Earthquake(datetime=datetime.datetime(2024, 1, 15...
    """

    latest_earthquake_data = info_latest_earthquake_data["Infogempa"]["gempa"]
    earthquake = parse_earthquake_data(latest_earthquake_data)
    potency = latest_earthquake_data["Potensi"]
    felt = latest_earthquake_data["Dirasakan"]
    shakemap = latest_earthquake_data["Shakemap"]

    return LatestEarthquake(earthquake, potency, felt, shakemap)


def parse_strong_earthquake_data(
    info_strong_earthquake_data: InfoStrongEarthquakeData,
) -> Iterator[StrongEarthquake]:
    """
    Parse `InfoStrongEarthquakeData` JSON.

    Args:
        info_strong_earthquake_data: A dict representing JSON of info strong earthquake.

    Yields:
        A `StrongEarthquake` schema.

    Examples:
        >>> info_strong_earthquake_data = InfoLatestEarthquakeData(
        ...     {
        ...         "Infogempa": {
        ...             "gempa": [
        ...                 {
        ...                     "Tanggal": "09 Jan 2024",
        ...                     "Jam": "12:25:24 WIB",
        ...                     "DateTime": "2024-01-09T05:25:24+00:00",
        ...                     "Coordinates": "-4.46,133.91",
        ...                     "Lintang": "4.46 LS",
        ...                     "Bujur": "133.91 BT",
        ...                     "Magnitude": "5.3",
        ...                     "Kedalaman": "10 km",
        ...                     "Wilayah": "130 km BaratDaya KAIMANA-PAPUABRT",
        ...                     "Potensi": "Tidak berpotensi tsunami",
        ...                 }
        ...             ]
        ...         }
        ...     }
        ... )
        >>> parse_strong_earthquake_data(info_strong_earthquake_data)
        <generator object parse_strong_earthquake_data at ...>
    """

    for strong_earthquake_data in info_strong_earthquake_data["Infogempa"]["gempa"]:
        earthquake = parse_earthquake_data(strong_earthquake_data)
        potency = strong_earthquake_data["Potensi"]

        yield StrongEarthquake(earthquake, potency)


def parse_felt_earthquake_data(
    info_felt_earthquake_data: InfoFeltEarthquakeData,
) -> Iterator[FeltEarthquake]:
    """
    Parse `InfoFeltEarthquakeData` JSON.

    Args:
        info_felt_earthquake_data: A dict representing JSON of info felt earthquake.

    Yields:
        A `FeltEarthquake` schema.

    Examples:
        >>> info_felt_earthquake_data = InfoFeltEarthquakeData(
        ...     {
        ...         "Infogempa": {
        ...             "gempa": [
        ...                 {
        ...                     "Tanggal": "15 Jan 2024",
        ...                     "Jam": "16:07:08 WIB",
        ...                     "DateTime": "2024-01-15T09:07:08+00:00",
        ...                     "Coordinates": "-3.98,122.45",
        ...                     "Lintang": "3.98 LS",
        ...                     "Bujur": "122.45 BT",
        ...                     "Magnitude": "2.6",
        ...                     "Kedalaman": "9 km",
        ...                     "Wilayah": "Pusat gempa berada di darat 2,5 km BaratDa",
        ...                     "Dirasakan": "II Kendari",
        ...                 }
        ...             ]
        ...         }
        ...     }
        ... )
        >>> parse_felt_earthquake_data(info_felt_earthquake_data)
        <generator object parse_felt_earthquake_data at ...>
    """

    for felt_earthquake_data in info_felt_earthquake_data["Infogempa"]["gempa"]:
        earthquake = parse_earthquake_data(felt_earthquake_data)
        felt = felt_earthquake_data["Dirasakan"]

        yield FeltEarthquake(earthquake, felt)



from ..bmkg import BMKG
from .enums import Province
from .parser import parse_weather_forecast_data
from .schemas import WeatherForecast as WeatherForecastData

__all__ = ["WeatherForecast"]


class WeatherForecast(BMKG):
    """
    Weather Forecast API Wrapper from BMKG API.
    """

    url = "/DataMKG/MEWS/DigitalForecast"

    async def get_weather_forecast(self, province: Province) -> WeatherForecastData:
        """
        Request weather forecast from weather forecast API.

        Args:
            province: A `Province` enum symbolic names (members).

        Returns:
            A `WeatherForecastData` schema.

        Examples:
            >>> import asyncio
            >>> async def main():
            ...     async with WeatherForecast() as weather_forecast:
            ...         weather_forecast_data = await weather_forecast.get_weather_forecast(
            ...             Province.ACEH
            ...         )
            ...         print(weather_forecast_data)
            >>> asyncio.run(main())
            WeatherForecast(data=Data(source=...)
        """  # noqa: E501
        async with self._session.get(
            f"{self.url}/DigitalForecast-{province}.xml"
        ) as response:
            return parse_weather_forecast_data(await response.read())



from dataclasses import dataclass

__all__ = ["Name"]


@dataclass(unsafe_hash=True, slots=True)
class Name:
    """
    A schema used to store info about name of an area.

    Attributes:
        en_US: name of an area in english.
        id_ID: name of an area in indonesian.
    """

    en_US: str
    id_ID: str



from collections.abc import Iterator

from ..bmkg import BMKG
from .parser import (
    parse_felt_earthquake_data,
    parse_latest_earthquake_data,
    parse_strong_earthquake_data,
)
from .schemas import (
    FeltEarthquake,
    LatestEarthquake,
    StrongEarthquake,
)
from .types import Shakemap

__all__ = ["Earthquake"]


class Earthquake(BMKG):
    """
    Earthquake API Wrapper from BMKG API.
    """

    url = "/DataMKG/TEWS"

    async def get_latest_earthquake(self) -> LatestEarthquake:
        """
        Request latest earthquake from earthquake API.

        Returns:
            A `LatestEarthquake` schema.

        Examples:
            >>> import asyncio
            >>> async def main():
            ...     async with Earthquake() as earthquake:
            ...         latest_earthquake = await earthquake.get_latest_earthquake()
            ...         print(latest_earthquake)
            >>> asyncio.run(main())
            LatestEarthquake(earthquake=Earthquake(datetime=datetime.datetime(...)
        """
        async with self._session.get(f"{self.url}/autogempa.json") as response:
            return parse_latest_earthquake_data(await response.json())

    async def get_latest_earthquake_shakemap(self, shakemap: Shakemap) -> bytes:
        """
        Request latest earthquake shakemap from earthquake API.

        Args:
            shakemap: A `Shakemap` file name.

        Returns:
            A bytes of `Shakemap` image.

        Examples:
            >>> import asyncio
            >>> async def main():
            ...     async with Earthquake() as earthquake:
            ...         latest_earthquake = await earthquake.get_latest_earthquake()
            ...         shakemap = await earthquake.get_latest_earthquake_shakemap(
            ...             latest_earthquake.shakemap
            ...         )
            ...         print(shakemap)
            >>> asyncio.run(main())
            b'...'
        """
        async with self._session.get(f"{self.url}/{shakemap}") as response:
            return await response.read()

    async def get_strong_earthquake(self) -> Iterator[StrongEarthquake]:
        """
        Request strong earthquake that has magnitude 5.0 above from earthquake API.

        Returns:
            An iterator of fifteen `StrongEarthquake` schema.

        Examples:
            >>> import asyncio
            >>> async def main():
            ...     async with Earthquake() as earthquake:
            ...         strong_earthquake = await earthquake.get_strong_earthquake()
            ...         print(strong_earthquake)
            >>> asyncio.run(main())
            <generator object parse_strong_earthquake_data at ...>
        """
        async with self._session.get(f"{self.url}/gempaterkini.json") as response:
            return parse_strong_earthquake_data(await response.json())

    async def get_felt_earthquake(self) -> Iterator[FeltEarthquake]:
        """
        Request felt earthquake from earthquake API.

        Returns:
            An iterator of fifteen `FeltEarthquake` schema.

        Examples:
            >>> import asyncio
            >>> async def main():
            ...     async with Earthquake() as earthquake:
            ...         felt_earthquake = await earthquake.get_felt_earthquake()
            ...         print(felt_earthquake)
            >>> asyncio.run(main())
            <generator object parse_felt_earthquake_data at ...>
        """
        async with self._session.get(f"{self.url}/gempadirasakan.json") as response:
            return parse_felt_earthquake_data(await response.json())



from .earthquake import Earthquake
from .felt_earthquake import FeltEarthquake
from .latest_earthquake import LatestEarthquake
from .strong_earthquake import StrongEarthquake

__all__ = [
    "Earthquake",
    "FeltEarthquake",
    "LatestEarthquake",
    "StrongEarthquake",
]



from .area import Area
from .data import Data
from .forecast import Forecast
from .humidity import Humidity
from .name import Name
from .temperature import Temperature
from .weather import Weather
from .weather_forecast import WeatherForecast
from .wind_direction import WindDirection
from .wind_speed import WindSpeed

__all__ = [
    "Area",
    "Data",
    "Forecast",
    "Humidity",
    "Name",
    "Temperature",
    "WeatherForecast",
    "Weather",
    "WindDirection",
    "WindSpeed",
]



from dataclasses import dataclass, field

from ..enums import Cardinal, Sexa

__all__ = ["WindDirection"]


@dataclass(slots=True)
class WindDirection:
    """
    A schema used to store info about wind direction.

    Attributes:
        deg: degree of a wind direction with deg as its unit.
        card: card direction of a wind direction with CARD as its unit.
        sexa: sexa direction of a wind direction with SEXA as its unit.
    """

    deg: float = field(metadata={"unit": "deg"})
    card: Cardinal = field(metadata={"unit": "CARD"})
    sexa: Sexa = field(metadata={"unit": "SEXA"})



"""
Module to store a str enum class representation area type.
"""
from enum import StrEnum

__all__ = ["Type"]


class Type(StrEnum):
    """
    A str enum class that define valid area type.

    Attributes:
        LAND: `"land"`
        SEA: `"sea"`

    Examples:
        >>> Type("land")
        <Type.LAND: 'land'>
        >>> Type["LAND"]
        <Type.LAND: 'land'>
        >>> Type.LAND
        <Type.LAND: 'land'>
        >>> Type.LAND == "land"
        True
        >>> print(Type.LAND)
        land
    """

    LAND: str = "land"
    SEA: str = "sea"



from dataclasses import dataclass, field

__all__ = ["Temperature"]


@dataclass(slots=True)
class Temperature:
    """
    A schema used to store info about temperature.

    Attributes:
        celcius: temperature in celcius with C as its unit.
        fahrenheit: temperature in fahrenheit with F as its unit.
    """

    celcius: float = field(metadata={"unit": "C"})
    fahrenheit: float = field(metadata={"unit": "F"})
